var Router = require('@koa/router');
var Koa = require('koa');
var onFinished = require('on-finished');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

var Router__default = /*#__PURE__*/_interopDefaultLegacy(Router);
var Koa__default = /*#__PURE__*/_interopDefaultLegacy(Koa);
var onFinished__default = /*#__PURE__*/_interopDefaultLegacy(onFinished);

function _extends() {
  _extends = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };

  return _extends.apply(this, arguments);
}

class KoaApi extends Koa__default["default"] {
  constructor(options = {}) {
    var _options$attachBody;

    super(options.koa);
    this.router = void 0;
    this.firstRun = true;
    this.options = void 0;
    this.options = _extends({}, options, {
      attachBody: (_options$attachBody = options == null ? void 0 : options.attachBody) != null ? _options$attachBody : true
    });
    this.router = new Router__default["default"](this.options.router);

    if (this.options.attachBody) {
      this.use(async (ctx, next) => {
        ctx.request.body = ctx.req.body;
        return next();
      });
    }
  }
  /**
   * Attaches router to a particular path. Internally it setups a prefix on the router, and calls `router.routes()` and `router.allowedMethods()`
   * @param path - path to attach the router to
   * @param router - router to attach
   */


  attachRouter(path, router) {
    this.use(router.prefix(path).routes()).use(router.allowedMethods());
    return this;
  }
  /**
   * Run koa api with request and response objects
   * @param req - nextjs request
   * @param res - nextjs response
   */


  async run(req, res) {
    const p = new Promise(resolve => onFinished__default["default"](res, resolve));

    if (this.firstRun) {
      this.firstRun = false;
      this.use(this.router.routes()).use(this.router.allowedMethods(this.options.routerAllowedMethods));
    }

    this.callback()(req, res);
    return p;
  }

}
/**
 * Helper function for Next.js api routes that automatically runs the KoaApi.
 * It should be used as default export from the Next.js api route file
 * @param koa - KoaApi to run
 */

function withKoaApi(koa) {
  return (req, res) => koa.run(req, res);
}

Object.defineProperty(exports, 'Router', {
  enumerable: true,
  get: function () { return Router__default["default"]; }
});
Object.defineProperty(exports, 'Koa', {
  enumerable: true,
  get: function () { return Koa__default["default"]; }
});
exports.KoaApi = KoaApi;
exports.withKoaApi = withKoaApi;
//# sourceMappingURL=index.js.map
